
var lpadAlign = require('lpad-align')
var longest = require('longest')
var chalk = require('chalk')
var tsml = require('tsml')

var isPlainObj = function (o) { return o !== null && typeof o === 'object' && o.constructor === Object; }

var formatValue = function (colors, val) {
  if (typeof val === 'number') {
    return colors.number(val)
  } else if (typeof val === 'boolean' || val === null) {
    return colors.literal(val)
  } else if (isPlainObj(val)) {
    return tsml(["\n      ", "\n      ", "\n      ", "\n    "], colors.punctuation('('), colors.string('collapsed'), colors.punctuation(')'))
  } else {
    return tsml(["\n      ", "\n      ", "\n      ", "\n    "], colors.punctuation('"'), colors.string(val), colors.punctuation('"'))
  }
}

var ifElse = function (bool, then, or) { return bool ? then() : or(); }

var format = function (
  obj,
  depth,
  colors,
  offset
) {
  if ( depth === void 0 ) depth = Infinity;
  if ( colors === void 0 ) colors = {
    punctuation: chalk.yellow,
    property: chalk.green,
    literal: chalk.magenta,
    number: chalk.cyan,
    string: chalk.bold
  };
  if ( offset === void 0 ) offset = 2;

  var keys = Object.keys(obj)
  var coloredKeys = keys.map(function (key) { return colors.property(key); })
  var colon = ': '

  if (!isPlainObj(depth)) {
    depth = { curr: 0, max: depth }
  }

  var parts = keys.map(function (key, i) {
    var val = obj[key]
    return tsml(["\n      ", "\n      ", "\n      ", "\n    "], lpadAlign(coloredKeys[i], coloredKeys, offset), colors.punctuation(colon), ifElse(
        depth.curr < depth.max && isPlainObj(val),
        function () { return format(
          val,
          { curr: depth.curr + 1, max: depth.max },
          colors,
          offset + longest(keys).length + colon.length
        ); },
        function () { return formatValue(colors, val); }
      ))
  })

  return '\n' + parts.join('\n')
}

module.exports = format

